/*_############################################################################
  _## 
  _##  SNMP4J-AgentJMX - MBeanAttributeMOTableInfo.java  
  _## 
  _##  Copyright (C) 2006-2009  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/

package org.snmp4j.agent.mo.jmx;

import java.io.IOException;
import javax.management.*;
import org.snmp4j.agent.mo.jmx.types.*;

/**
 * The <code>MBeanAttributeMOTableInfo</code> class describes the mapping from
 * the attributes of an MBean to a SNMP conceptual table and vice versa.
 *
 * @author Frank Fock
 * @version 1.0
 */
public class MBeanAttributeMOTableInfo extends MBeanMOInfo {

  private MBeanAttributeKeyProvider keyProvider;
  private String[] indexAttributes;
  private TypedAttribute[] columns;
  private JMXIndexSupport indexSupport;

  /**
   * Creates a table mapping for the supplied MBean name.
   *
   * @param name
   *    the name of the MBean that provides the table data.
   * @param keyProvider
   *    a key provider that provides the keys that identify rows in the table.
   * @param columns
   *    the attributes that represent the columns of the table.
   * @param keyAttributes
   *    the name of the attributes in <code>columns</code> that represent the
   *    primary key of the row.
   * @param indexSupport
   *    provides the mapping between the row keys and their corresponding
   *    SNMP index values.
   */
  public MBeanAttributeMOTableInfo(ObjectName name,
                                   MBeanAttributeKeyProvider keyProvider,
                                   TypedAttribute[] columns,
                                   String[] keyAttributes,
                                   JMXIndexSupport indexSupport) {
    super(name);
    this.keyProvider = keyProvider;
    this.indexAttributes = keyAttributes;
    this.columns = columns;
    this.indexSupport = indexSupport;
  }

  public String[] getIndexAttributes() {
    return indexAttributes;
  }

  public TypedAttribute[] getColumns() {
    return columns;
  }

  public MBeanAttributeKeyProvider getKeyProvider() {
    return keyProvider;
  }

  public JMXIndexSupport getIndexSupport() {
    return indexSupport;
  }

  public Object getKey(MBeanServerConnection server, ObjectName row) throws
      IOException, ReflectionException, InstanceNotFoundException {
    Object[] key = new Object[indexAttributes.length];
    AttributeList keyObjects = server.getAttributes(row, indexAttributes);
    for (int i=0; i<keyObjects.size(); i++) {
      key[i] = ((Attribute)keyObjects.get(i)).getValue();
    }
    if (key.length == 1) {
      return key[0];
    }
    return key;
  }
}
